from .model_for_question_answer import QuestionAnswerModel
from .model_for_text_classification import TextClassificationModel
from .model_for_token_classification import TokenClassificationModel
from .model_for_lora import LoraModel
from .model_for_chinese_alpaca import ChineseAlpacaModel
from .model_for_baichuan import BaichuanModel
from .model_for_chatglm import ChatglmModel
from .model_for_Ziya_llama_13b import ZiyallamaModel
