from .question_answer_pc import QuestionAnswerPreProcessor
from .text_classification_pc import TextClassificationPreProcessor
from .pc_for_alpaca import AlpacaPreProcessor
from .pc_for_llama_cn import VicunaPreProcessor
from .pc_for_chinese_alpaca import ChineseAlpacaPreProcessor
from .pc_for_baichuan import BaichuanPreProcessor
from .pc_for_chatglm import ChatglmPreProcessor
from .pc_for_ziya_llama_13b import Ziyallama13bPreProcessor
